#include "Aabb.h"

#include <cmath>
#include <algorithm>

void Aabb::extendBy(
    const Vec4f   &v3Point )  
{
    if( v3Point[0] < m_v3Min[0] )
        m_v3Min[0] = v3Point[0];
    if( v3Point[1] < m_v3Min[1] )
        m_v3Min[1] = v3Point[1];
    if( v3Point[2] < m_v3Min[2] )
        m_v3Min[2] = v3Point[2];
    if( v3Point[0] > m_v3Max[0] )
        m_v3Max[0] = v3Point[0];
    if( v3Point[1] > m_v3Max[1] )
        m_v3Max[1] = v3Point[1];
    if( v3Point[2] > m_v3Max[2] )
        m_v3Max[2] = v3Point[2];
}

void Aabb::extendBy(
    const Aabb    &clBox )    
{
    extendBy( clBox.getMin() );
    extendBy( clBox.getMax() );
}

bool Aabb::intersect(
    const Ray   &clRay, float &t0, float &t1 )  const
{
    const Vec4f   &v3_o = clRay.origin;
    const Vec4f   &v3_dir = clRay.direction;
    Vec4f         v3_near;
    Vec4f         v3_far;
    if( v3_dir[0] > 0 )
    {
        v3_near[0] = ( m_v3Min[0] - v3_o[0] ) / v3_dir[0];
        v3_far[0] = ( m_v3Max[0] - v3_o[0] ) / v3_dir[0];
    }
    else if( v3_dir[0] < 0 )
    {
        v3_near[0] = ( m_v3Max[0] - v3_o[0] ) / v3_dir[0];
        v3_far[0] = ( m_v3Min[0] - v3_o[0] ) / v3_dir[0];
    }
    else
    {
        v3_near[0] = -FLT_MAX;
        v3_far[0] = FLT_MAX;
    }

    if( v3_dir[1] > 0 )
    {
        v3_near[1] = ( m_v3Min[1] - v3_o[1] ) / v3_dir[1];
        v3_far[1] = ( m_v3Max[1] - v3_o[1] ) / v3_dir[1];
    }
    else if( v3_dir[1] < 0 )
    {
        v3_near[1] = ( m_v3Max[1] - v3_o[1] ) / v3_dir[1];
        v3_far[1] = ( m_v3Min[1] - v3_o[1] ) / v3_dir[1];
    }
    else
    {
        v3_near[1] = -FLT_MAX;
        v3_far[1] = FLT_MAX;
    }

    if( v3_dir[2] > 0 )
    {
        v3_near[2] = ( m_v3Min[2] - v3_o[2] ) / v3_dir[2];
        v3_far[2] = ( m_v3Max[2] - v3_o[2] ) / v3_dir[2];
    }
    else if( v3_dir[2] < 0 )
    {
        v3_near[2] = ( m_v3Max[2] - v3_o[2] ) / v3_dir[2];
        v3_far[2] = ( m_v3Min[2] - v3_o[2] ) / v3_dir[2];
    }
    else
    {
        v3_near[2] = -FLT_MAX;
        v3_far[2] = FLT_MAX;
    }

    t0 = std::max( v3_near[0], std::max( v3_near[1], v3_near[2] ) );
    t1 = std::min( v3_far[0], std::min( v3_far[1], v3_far[2] ) );

    return t0 < t1;
}

bool Aabb::intersect( const Aabb &other ) const
{
    return (getMin()[0]-0.0001 <= other.getMax()[0] && getMax()[0]+0.0001 >= other.getMin()[0])
        && (getMin()[1]-0.0001 <= other.getMax()[1] && getMax()[1]+0.0001 >= other.getMin()[1])
        && (getMin()[2]-0.0001 <= other.getMax()[2] && getMax()[2]+0.0001 >= other.getMin()[2]);
}

float Aabb::distance( const Aabb &other ) const
{
    float dx = std::max<float>(0, std::max<float>(m_v3Min.x-other.m_v3Max.x, other.m_v3Min.x-m_v3Max.x));
    float dy = std::max<float>(0, std::max<float>(m_v3Min.y-other.m_v3Max.y, other.m_v3Min.y-m_v3Max.y));
    float dz = std::max<float>(0, std::max<float>(m_v3Min.z-other.m_v3Max.z, other.m_v3Min.z-m_v3Max.z));
    return sqrtf(dx*dx + dy*dy + dz*dz);
}