#pragma once

#include "Ray.h"

class Aabb
{

    //-------------------------------------------------------------- private data
private:
    Vec4f m_v3Min;
    Vec4f m_v3Max;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Aabb() :
    m_v3Min( FLT_MAX, FLT_MAX, FLT_MAX ),
    m_v3Max( -FLT_MAX, -FLT_MAX, -FLT_MAX )
    {
    };

    //! initializing constructor
    Aabb(
        const Vec4f   &v3Min, 
        const Vec4f   &v3Max )  :
    m_v3Min( v3Min ),
    m_v3Max( v3Max )
    {
    };

    //! destructor	
    ~Aabb()
    {
    };

    //----------------------------------------------------------- element access
    void setMin(
        const Vec4f   &v3Min )    
    {
        m_v3Min = v3Min;
    };

    
    const Vec4f &getMin() const
    {
        return m_v3Min;
    };

    
    void setMax(
        const Vec4f   &v3Max )    
    {
        m_v3Max = v3Max;
    };

    
    const Vec4f &getMax() const
    {
        return m_v3Max;
    };

    //----------------------------------------------------------- operators
    Aabb &operator+=(
        const Vec4f   &v3Offset ) 
    {
        m_v3Min += v3Offset;
        m_v3Max += v3Offset;
        return ( *this );
    };

    
    Aabb &operator*=(
        const Vec4f   &v3Scale )  
    {
        m_v3Min *= v3Scale;
        m_v3Max *= v3Scale;
        return ( *this );
    };

    
    Aabb &operator*=(
        const float  fScale )    
    {
        m_v3Min *= fScale;
        m_v3Max *= fScale;
        return ( *this );
    };

    //----------------------------------------------------------- functionality
public:
    
    const Vec4f getCenter()
    {
        auto ret = ( m_v3Min + m_v3Max ) * 0.5;
        return Vec4f( ret.x, ret.y, ret.z, 1 );
    };

    
    const Vec4f getSize()
    {
        return ( m_v3Max - m_v3Min );
    };

    
    void Reset()
    {
        m_v3Min[0] = m_v3Min[1] = m_v3Min[2] = FLT_MAX;
        m_v3Max[0] = m_v3Max[1] = m_v3Max[2] = -FLT_MAX;
    };

    //!Extend by point.
    void    extendBy( const Vec4f &v3Point );

    //!Extend by box.
    void    extendBy( const Aabb &clBox );

    //!	intersect?
    bool    intersect( const Ray &clRay, float &t0, float &t1 ) const;
    bool    intersect( const Aabb &other ) const;

    float   distance( const Aabb &other ) const;
};