#pragma once

#include "Intersection.h"
#include "Ray.h"

class SurfaceList;

/**
 * @brief Accelerator virtual class.
 * 
 * This class defines the basic operations that the explicit Acceleration
 * Data Structure (ADS) should provide. This class therefore only provides
 * the necessary members to interface with the Raytracer, you need to add
 * new member(s) and also likely a class for nodes when implementing an
 * ADS that is based on a tree.
 */
class Accelerator
{
public:
    /**
     * @brief Initialize the Accelerator based on the scene's surfaces.
	 * 
	 * Initialize the Accelerator based on the scene's surfaces, which are
	 * accessed by scene->getSurfaces(). Every Surface needs to be considered.
	 * However, Planes and Checkboards might need to be handled with an
	 * exception, as their AABB is infinitely big, and you don't want to span
	 * your ADS across infinite space.
     * 
     * @param scene The surfaces that make up the scene and should be
     *              represented by the acceleration data structure.
     */
	Accelerator(const SurfaceList *scene) {};

    /**
     * @brief Release all resources that might have been requested in
	 * 		  the accelerator's constructor.
     */
	virtual ~Accelerator() {};

    /**
     * @brief Intersect the scene, accelerated by the ADS, by reducing
	 * 		  the amount of ray-surface intersection tests.
     * 
     * @param ray The ray that is to intersected with the Accelerator & the scene
     * @param tStart The minimal distance for a valid hit (along the ray, normalized).
     * @param tEnd The maximal distance for a valid hit (along the ray, normalized).
     */
    virtual Intersection intersect( Ray ray, float tStart, float tEnd ) const = 0;
};