#include "Camera.h"


Camera::Camera(){
    target = Vec4f();
    distance = 3.f;
    focalLength = 1.f;
    updateModelMatrix();
}

Camera::Camera(Vec4f pTarget, float pDistance, float pFocalLength, float xRotation, float yRotation){
    target = pTarget;
    distance = pDistance;
    focalLength = pFocalLength;

    mouseX = yRotation * 100;
    mouseY = xRotation * 100;

    updateModelMatrix();
}

Ray Camera::generateRay(unsigned int x, unsigned int y, unsigned int imageWidth, unsigned int imageHeight){
    // Calculate the direction of the ray (assuming that (1,1,1) and (-1,-1,1) are the min and max values
    // of the upper left and the lower right image):
    Vec4f direction(
        -1 + 2 * ((x + 0.5f) / (imageWidth - 1)),
        1 - 2 * ((y + 0.5f) / (imageHeight - 1)),
        -focalLength,
        0
    );

    // Transform the origin and direction into world space and return it in the Ray struct:
    return Ray(model * target, model * direction);
}

void Camera::rotateCamera(float x, float y){
    mouseX += x;
    mouseY += y;
    updateModelMatrix();
}

void Camera::multiplyDistance(float multiplier){
    distance *= multiplier;
    updateModelMatrix();
}

void Camera::updateModelMatrix(){
    model = Mat4f::rotationY(-mouseX * 0.01f) * Mat4f::rotationX(-mouseY*0.01f) * Mat4f::translation(0.0f, 0.0f, distance);
}
