// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "src/raytracer/Ray.h"
#include "src/raytracer/Intersection.h"

#include "src/shader/Shader.h"
#include "Aabb.h"

/**
 * An abstract surface definition. A surface can consist of several
 * non-contiguous parts, e.g. via the SurfaceList class, which inherits
 * from this Surface class. It can also be a Sphere, Plane, Checkerboard,
 * etc. (see 'surfaces' folder).
 */
class Surface {
protected:
    Shader shader;

public:
    /** Destructor */
    virtual ~Surface(){};

    /**
     * Performs an intersection check between the specified ray and the
     * given surface, returning an intersection only if the intersection
     * is between the [tStart, tEnd] section of the ray.
     *
     * The parameters tStart and tEnd are the line parameter t of the
     * following equation:
     *
     * point = ray.origin + ray.direction * t
     *
     * @param ray       The given ray.
     * @param tStart    The start parameter.
     * @param tEnd      The end parameter.
     *
     * @return The result of the intersection test.
     */
    virtual Intersection intersect(Ray ray, float tStart, float tEnd) = 0;

    virtual Aabb getAabb() const = 0;
};
