#include "Checkerboard.h"

Checkerboard::Checkerboard(Vec4f pPoint, Vec4f pX, Vec4f pY, Shader pShader, Shader pShader2){
    point = pPoint;
    X = pX;
    Y = pY;
    normal = (X - point).cross(Y - point).normalized();
    shader = pShader;
    shader2 = pShader2;
}

Intersection Checkerboard::intersect(Ray ray, float tStart, float tEnd){
    // Initialize result struct (per default, hasIntersection is false):
    Intersection result;

    float f_denominator = normal.dot(ray.direction);

    if ( fabs( f_denominator ) < FLT_EPSILON )
        return result;

    float t = normal.dot(point - ray.origin) / f_denominator;

    if ( t > tStart && t < tEnd ){
        result.hasIntersection = true;
        result.t = t;
        result.hitPoint = ray.origin + ray.direction * t;
        result.normal = normal;

        Vec4f inPlane = result.hitPoint - point;
        float uvCoordX = (X - point).dot(inPlane);
        float uvCoordY = (Y - point).dot(inPlane);

        int quantUVCoordX = abs(int(uvCoordX)) + (uvCoordX < 0 ? 1 : 0);
        int quantUVCoordY = abs(int(uvCoordY)) + (uvCoordY < 0 ? 1 : 0);

        if(quantUVCoordX % 2 == 0 != quantUVCoordY % 2 == 0){
            result.shader = &shader;
        } else {
            result.shader = &shader2;
        }
    }

    return result;
}

Aabb Checkerboard::getAabb() const
{
    return Aabb();
}