#include "Sphere.h"

Sphere::Sphere(Vec4f pCenter, float pRadius, Shader pShader){
    center = pCenter;
    radius = pRadius;
    shader = pShader;
}

Intersection Sphere::intersect(Ray ray, float tStart, float tEnd){
    // Initialize result struct (per default, hasIntersection is false):
    Intersection result;

    Vec4f originDistance = ray.origin - center;

    // Coefficients of the quadratic equation:
    double A = ray.direction.dot(ray.direction);
    double B = 2 * ray.direction.dot(originDistance);
    double C = originDistance.dot(originDistance) - radius * radius;

    double discriminant = B*B-4*A*C;
    if ( discriminant > FLT_EPSILON )
    {
        double sqrtdis = sqrt( discriminant );

        // Check for the first possible solution:
        float t = float(( -B - sqrtdis ) / ( 2*A ));

        if(t > tStart && t < tEnd)
        {
            result.hasIntersection = true;
            result.t = t;
            result.hitPoint = ray.origin + ray.direction * t;
            result.normal = (result.hitPoint - center) * (1/radius);
            result.shader = &shader;
            return result;
        }

        // Check for the second possible solution:
        t = float((-B + sqrtdis) / (2*A));
        if(t > tStart && t < tEnd)
        {
            result.hasIntersection = true;
            result.t = t;
            result.hitPoint = ray.origin + ray.direction * t;
            result.normal = (result.hitPoint - center) * (1/radius);
            result.shader = &shader;
            return result;
        }
    }

    return result;
}

Aabb Sphere::getAabb() const
{
    Vec4f radius_vector(radius, radius, radius, 0);
    return Aabb(center-radius_vector, center+radius_vector);
}