// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "src/scene/Surface.h"

#include "src/shader/Shader.h"

class Sphere : public Surface {
    /** Center position of the sphere */
    Vec4f center;

    /** Radius of the sphere */
    float radius;

    /** Shader that is used to shade a pixel */
    Shader shader;

public:
    /**
     * Creates a sphere at the given position, with the given radius and the given shader.
     *
     * @param pCenter   Center position of the sphere.
     * @param pRadius   Radius
     * @param pShader   Shader.
     */
    Sphere(Vec4f pCenter, float pRadius, Shader pShader);

    /** See documentation in parent class 'Surface' */
    virtual Intersection intersect(Ray ray, float tStart, float tEnd) override;

    /** See documentation in parent class 'Surface' */
    virtual Aabb getAabb() const override;
};
